use bevy::{prelude::*, window::PrimaryWindow};

pub struct Plugin;
impl bevy::prelude::Plugin for Plugin {
    fn build(&self, app: &mut App) {
        app.add_systems(Startup, setup)
            .add_systems(Update, move_cam)
            .init_resource::<MousePos>();
    }
}

fn setup(mut cmds: Commands, window: Query<&Window>) {
    let zoom = 2./window.single().width().min(window.single().height());
    cmds.spawn((
        Camera2d,
        Transform::from_scale(Vec3::new(zoom, zoom, 1.))
    ));
}

#[derive(Resource, Default)]
struct MousePos(Vec2);

fn move_cam(
    mut cam: Query<&mut Transform, With<Camera2d>>,
    mouse_buttons: Res<ButtonInput<MouseButton>>,
    window: Query<&Window, With<PrimaryWindow>>,
    mut mouse_pos: ResMut<MousePos>
) {
    let window = window.single();
    if let Some(cursor_pos) = window.cursor_position() {
        if mouse_buttons.pressed(MouseButton::Left) {
            let delta = cursor_pos - mouse_pos.0;
            dbg!(delta);
            let mut cam = cam.single_mut();
            cam.translation.x -= delta.x*cam.scale.x;
            cam.translation.y += delta.y*cam.scale.y;
        }
        mouse_pos.0 = cursor_pos;
    }
}